import os
from pathlib import Path
from typing import Final

import typer
from beni import bcolor, bfile, bpath, brun, btask
from beni.bfunc import syncCall

from bcmdx.utils.tkUtil import TkForm

from ..common import password

app: Final = btask.newSubApp('lib 工具')


@app.command()
@syncCall
async def update_version(
    path: Path = typer.Argument(Path.cwd(), help='workspace 路径'),
    isNotCommit: bool = typer.Option(False, '--no-commit', '-d', help='是否提交git'),
):
    '修改 pyproject.toml 版本号'
    file = path / 'pyproject.toml'
    btask.assertTrue(file.is_file(), '文件不存在', file)
    data = await bfile.readToml(file)
    latestVersion = data['project']['version']
    vAry = [int(x) for x in latestVersion.split('.')]
    versionList = [
        f'{vAry[0] + 1}.0.0',
        f'{vAry[0]}.{vAry[1] + 1}.0',
        f'{vAry[0]}.{vAry[1]}.{vAry[2] + 1}',
    ]

    def showGui():
        app = TkForm()
        app.title('bcmd 版本更新')
        app.addLabel('当前版本号', latestVersion)
        version_var = app.addRadioBtn(
            '请选择新版本',
            versionList,
            selectedIndex=-1,
        )
        result: str = ''

        def onBtn():
            nonlocal result
            result = version_var.get()
            app.destroy()

        app.addBtn('确定', onBtn, focus=True)
        app.run()
        return result

    newVersion = showGui()
    if not newVersion:
        btask.abort('用户取消操作')
    content = await bfile.readText(file)
    if f"version = '{latestVersion}'" in content:
        content = content.replace(f"version = '{latestVersion}'", f"version = '{newVersion}'")
    elif f'version = "{latestVersion}"' in content:
        content = content.replace(f'version = "{latestVersion}"', f'version = "{newVersion}"')
    else:
        raise Exception('版本号修改失败，先检查文件中定义的版本号格式是否正常')
    await bfile.writeText(file, content)

    # 执行一遍 uv.lock
    with bpath.changePath(path):
        await brun.run('uv lock')

    bcolor.printCyan(newVersion)
    if not isNotCommit:
        msg = f'更新版本号 {newVersion}'
        os.system(
            rf'TortoiseGitProc.exe /command:commit /path:{path} /logmsg:"{msg}"'
        )
    bcolor.printGreen('OK')


@app.command()
@syncCall
async def build(
    path: Path = typer.Argument(Path.cwd(), help='workspace 路径'),
):
    '发布项目'
    user, pwd = await password.getPypi()
    bpath.remove(path / 'dist')
    bpath.remove(
        *list(path.glob('*.egg-info'))
    )
    with bpath.changePath(path):
        await brun.run(f'uv build')
        await brun.run(f'uv publish -u {user} -p {pwd}')
    bcolor.printGreen('OK')
